@@ -76,7 +76,7 @@ void transmit_answer(unsigned char seqnum,unsigned int len)
 
 void programcmd(unsigned char seqnum)
 {
-        unsigned char tmp,tmp2,addressing_is_word,ci,cj;
+        unsigned char tmp,tmp2,addressing_is_word,ci,cj,cstatus;
         unsigned int answerlen;
         unsigned long poll_address=0;
         unsigned int i,nbytes;
@@ -112,7 +112,6 @@ void programcmd(unsigned char seqnum)
                 break;
 
                 case CMD_GET_PARAMETER:
-                cbi(PORTB,PB2); //dbg
                 tmp=0xff;
                 switch(msg_buf[1])
                 {
@@ -213,6 +212,10 @@ void programcmd(unsigned char seqnum)
                 if (msg_buf[4]> 32){
                         msg_buf[4]=32;
                 }
+                if (msg_buf[5] < 3){
+                        // minimum delay
+                        msg_buf[5]=3;
+                }
                 while(i<msg_buf[4]){//synchLoops
                         wd_kick();
                         LED_ON; // blink during init
@@ -259,13 +262,15 @@ void programcmd(unsigned char seqnum)
                 spi_mastertransmit(msg_buf[4]);
                 spi_mastertransmit(msg_buf[5]);
                 spi_mastertransmit(msg_buf[6]);
-
                 if(msg_buf[2]==0) {
                         // pollMethod use delay
                         delay_ms(msg_buf[1]); // eraseDelay
                 } else {
                         // pollMethod RDY/BSY cmd
-                        while(spi_mastertransmit_32(0xF0000000)&1);
+                        ci=150; // timeout
+                        while((spi_mastertransmit_32(0xF0000000)&1)&&ci){
+                                ci--;
+                        }
                 }
                 answerlen = 2;
                 //msg_buf[0] = CMD_CHIP_ERASE_ISP;
@@ -275,7 +280,7 @@ void programcmd(unsigned char seqnum)
                 case CMD_PROGRAM_EEPROM_ISP:
                 addressing_is_word=0;  // address each byte
                 case CMD_PROGRAM_FLASH_ISP:
-                // msg_buf[0] CMD_PROGRAM_FLASH_ISP
+                // msg_buf[0] CMD_PROGRAM_FLASH_ISP = 0x13
                 // msg_buf[1] NumBytes H
                 // msg_buf[2] NumBytes L
                 // msg_buf[3] mode
@@ -286,17 +291,30 @@ void programcmd(unsigned char seqnum)
                 // msg_buf[8] poll1 (value to poll)
                 // msg_buf[9] poll2
                 // msg_buf[n+10] Data
+                poll_address=0;
+                // set a minimum delay
+                if (msg_buf[4] < 3){
+                        msg_buf[4]=3;
+                }
+                // set a max delay
+                if (msg_buf[4] > 24){
+                        msg_buf[4]=24;
+                }
 		saddress=address; // previous address, start address 
-                nbytes=((int)msg_buf[1])<<8; //NumBytes
-                nbytes|= msg_buf[2];
+		nbytes = ((unsigned int)msg_buf[1])<<8;
+		nbytes |= msg_buf[2];
                 if (nbytes> 280){
                         // corrupted message
                         answerlen = 2;
                         msg_buf[1] = STATUS_CMD_FAILED;
                         break;
                 }
-                // msg_buf[3] test Word/Page Mode bit:
                 wd_kick();
+                // store the original mode:
+                tmp2=msg_buf[3];
+                // result code
+                cstatus=STATUS_CMD_OK;
+                // msg_buf[3] test Word/Page Mode bit:
                 if ((msg_buf[3]&1)==0){
                         // word mode
                         for(i=0;i<nbytes;i++)
@@ -312,20 +330,28 @@ void programcmd(unsigned char seqnum)
                                 }
                                 spi_mastertransmit_16(address&0xFFFF);
                                 spi_mastertransmit(msg_buf[i+10]);
-                                // that the data byte is not same as poll value
+                                // if the data byte is not same as poll value
                                 // in that case we can do polling:
-                                if((msg_buf[8]!=msg_buf[i+10])) {
+                                if(msg_buf[8]!=msg_buf[i+10]) {
                                         poll_address = address&0xFFFF;
+                                        // restore the possibly modifed mode:
+                                        msg_buf[3]=tmp2;
                                 } else {
-                                        //switch the mode to timed delay (waiting)
+                                        //switch the mode to timed delay (waiting), for this word
                                         msg_buf[3]= 0x02;
                                 }
-                                
+                                //
+                                wd_kick();
+                                if (!addressing_is_word){
+                                        // eeprom writing, eeprom needs more time
+                                        delay_ms(1);
+                                }
                                 //check the different polling mode methods
                                 if(msg_buf[3]& 0x04) {
                                         //data value polling
                                         tmp=msg_buf[8];
-                                        while(tmp==msg_buf[8]){
+                                        ci=150; // timeout
+                                        while(tmp==msg_buf[8] && ci ){
                                                 // The Low/High byte selection bit is
                                                 // bit number 3. Set high byte for uneven bytes
                                                 // Read data:
@@ -336,10 +362,14 @@ void programcmd(unsigned char seqnum)
                                                 }
                                                 spi_mastertransmit_16(poll_address);
                                                 tmp=spi_mastertransmit(0x00);
+                                                ci--;
                                         }
                                 } else if(msg_buf[3]&0x08){
                                         //RDY/BSY polling
-                                        while(spi_mastertransmit_32(0xF0000000)&1);
+                                        ci=150; // timeout
+                                        while((spi_mastertransmit_32(0xF0000000)&1)&&ci){
+                                                ci--;
+                                        }
                                 }else{
                                         //timed delay (waiting)
                                         delay_ms(msg_buf[4]);
@@ -353,7 +383,7 @@ void programcmd(unsigned char seqnum)
                                 }
                         }                        
                 }else{
-                        //page mode
+                        //page mode, all modern chips
                         for(i=0;i<nbytes;i++)
                         {
                                 // The Low/High byte selection bit is
@@ -372,7 +402,8 @@ void programcmd(unsigned char seqnum)
                                         poll_address = address&0xFFFF;
                                 } else {
                                         //switch the mode to timed delay (waiting)
-                                        msg_buf[3] = (msg_buf[3]&(~0x70)) | 0x10;
+                                        //we must preserve bit 0x80
+                                        msg_buf[3]= (msg_buf[3]&0x80)|0x10;
                                 
                                 }
                                 if (addressing_is_word){
@@ -383,17 +414,26 @@ void programcmd(unsigned char seqnum)
                                 }
                         }
                         
+                        wd_kick();
+                        //page mode check result:
+                        //
                         // stk sets the Write page bit (7) if the page is complete
                         // and we should write it.
                         if(msg_buf[3]&0x80) {
                                 spi_mastertransmit(msg_buf[6]);
                                 spi_mastertransmit_16(saddress&0xFFFF);
                                 spi_mastertransmit(0);
+                                //
+                                if (!addressing_is_word){
+                                        // eeprom writing, eeprom needs more time
+                                        delay_ms(1);
+                                }
                                 //check the different polling mode methods
-                                if(msg_buf[3]&0x20) {
+                                ci=150; // timeout
+                                if(msg_buf[3]&0x20 && poll_address) {
                                         //Data value polling
                                         tmp=msg_buf[8];
-                                        while(tmp==msg_buf[8]){
+                                        while(tmp==msg_buf[8] && ci){
                                                 // The Low/High byte selection bit is
                                                 // bit number 3. Set high byte for uneven bytes
                                                 // Read data:
@@ -404,10 +444,19 @@ void programcmd(unsigned char seqnum)
                                                 }
                                                 spi_mastertransmit_16(poll_address);
                                                 tmp=spi_mastertransmit(0x00);
+                                                ci--;
+                                        }
+                                        if (ci==0){
+                                                cstatus=STATUS_CMD_TOUT;
                                         }
                                 } else if(msg_buf[3]& 0x40){
                                         //RDY/BSY polling
-                                        while(spi_mastertransmit_32(0xF0000000)&1);
+                                        while((spi_mastertransmit_32(0xF0000000)&1)&&ci){
+                                                ci--;
+                                        }
+                                        if (ci==0){
+                                                cstatus=STATUS_RDY_BSY_TOUT;
+                                        }
                                 }else{
                                         // simple waiting
                                         delay_ms(msg_buf[4]);
@@ -416,7 +465,7 @@ void programcmd(unsigned char seqnum)
                 }
                 answerlen = 2;
                 //msg_buf[0] = CMD_PROGRAM_FLASH_ISP; or CMD_PROGRAM_EEPROM_ISP
-                msg_buf[1] = STATUS_CMD_OK;
+                msg_buf[1] = cstatus;
                 break;
 
                 case CMD_READ_EEPROM_ISP:
@@ -424,7 +473,7 @@ void programcmd(unsigned char seqnum)
                 case CMD_READ_FLASH_ISP:
 		// msg_buf[1] and msg_buf[2] NumBytes
 		// msg_buf[3] cmd
-		nbytes = ((int)msg_buf[1])<<8;
+		nbytes = ((unsigned int)msg_buf[1])<<8;
 		nbytes |= msg_buf[2];
 		tmp = msg_buf[3];
                 // limit answer len, prevent overflow:
