@@ -21,6 +21,7 @@ static unsigned char sck_dur=1;
 //SPR0 and SPR1 =SPCR bit 0 and 1
 unsigned char  spi_set_sck_duration(unsigned char dur)
 {
+        /* nobody needs this slow speed:
         if (dur >= 3){
         	// 28KHz
         	cbi(SPSR,SPI2X);
@@ -29,7 +30,8 @@ unsigned char  spi_set_sck_duration(unsigned char dur)
                 sck_dur=3;
         	return(sck_dur);
         }
-        if (dur == 2){
+        */
+        if (dur >= 2){
         	// 57KHz
         	cbi(SPSR,SPI2X);
         	sbi(SPCR,SPR1);
