@@ -13,17 +13,18 @@
 #include "avr_compat.h"
 
 void delay_ms(unsigned int ms)
-/* delay for a minimum of <ms> */
-/* with a 4Mhz clock, the resolution is 1 ms */
+/* delay for about a minimum of <ms> */
+/* with a 4Mhz clock */
 {
         uint8_t inner,inner1;
         while (ms) {
-                inner = 200;
+                inner = 150;
                 while (inner) {
                         inner--;
-                        inner1 = 70;
+                        inner1 = 10;
                         while (inner1) {
                                 inner1--;
+                                asm("nop");
                         }
                 }
                 ms--;
